@@ -4,6 +4,7 @@
 #define   MYesNoAllRenQ      L"_Yes_No_All_nEver_Rename_Quit"
 #define   MContinueQuit      L"_Continue_Quit"
 #define   MRetryAbort        L"_Retry_Abort"
+#define   MIgnoreAllRetryQuit L"_Ignore_iGnore all_Retry_Quit"
 #define   MCopyright         L"\nRAR %s   Copyright (c) 1993-%d Alexander Roshal   %d %s %d"
 #define   MRegTo             L"\nRegistered to %s\n"
 #define   MShare             L"\nTrial version             Type 'rar -?' for help\n"
@@ -54,7 +55,7 @@
 #define   MCHelpSwm          L"\n  -             Stop switches scanning"
 #define   MCHelpSwAT         L"\n  @[+]          Disable [enable] file lists"
 #define   MCHelpSwAC         L"\n  ac            Clear Archive attribute after compression or extraction"
-#define   MCHelpSwAD         L"\n  ad            Append archive name to destination path"
+#define   MCHelpSwAD         L"\n  ad[1,2]       Alternate destination path"
 #define   MCHelpSwAG         L"\n  ag[format]    Generate archive name using the current date"
 #define   MCHelpSwAI         L"\n  ai            Ignore file attributes"
 #define   MCHelpSwAO         L"\n  ao            Add files with Archive attribute set"
@@ -79,13 +80,13 @@
 #define   MCHelpSwF          L"\n  f             Freshen files"
 #define   MCHelpSwHP         L"\n  hp[password]  Encrypt both file data and headers"
 #define   MCHelpSwHT         L"\n  ht[b|c]       Select hash type [BLAKE2,CRC32] for file checksum"
-#define   MCHelpSwIDP        L"\n  id[c,d,p,q]   Disable messages"
+#define   MCHelpSwIDP        L"\n  id[c,d,n,p,q] Display or disable messages"
 #define   MCHelpSwIEML       L"\n  ieml[addr]    Send archive by email"
 #define   MCHelpSwIERR       L"\n  ierr          Send all messages to stderr"
 #define   MCHelpSwILOG       L"\n  ilog[name]    Log errors to file"
 #define   MCHelpSwINUL       L"\n  inul          Disable all messages"
 #define   MCHelpSwIOFF       L"\n  ioff[n]       Turn PC off after completing an operation"
-#define   MCHelpSwISND       L"\n  isnd          Enable sound"
+#define   MCHelpSwISND       L"\n  isnd[-]       Control notification sounds"
 #define   MCHelpSwIVER       L"\n  iver          Display the version number"
 #define   MCHelpSwK          L"\n  k             Lock archive"
 #define   MCHelpSwKB         L"\n  kb            Keep broken extracted files"
@@ -127,11 +128,11 @@
 #define   MCHelpSwT          L"\n  t             Test files after archiving"
 #define   MCHelpSwTK         L"\n  tk            Keep original archive time"
 #define   MCHelpSwTL         L"\n  tl            Set archive time to latest file"
-#define   MCHelpSwTN         L"\n  tn<time>      Process files newer than <time>"
-#define   MCHelpSwTO         L"\n  to<time>      Process files older than <time>"
-#define   MCHelpSwTA         L"\n  ta<date>      Process files modified after <date> in YYYYMMDDHHMMSS format"
-#define   MCHelpSwTB         L"\n  tb<date>      Process files modified before <date> in YYYYMMDDHHMMSS format"
-#define   MCHelpSwTS         L"\n  ts[m|c|a]     Save or restore file time (modification, creation, access)"
+#define   MCHelpSwTN         L"\n  tn[mcao]<t>   Process files newer than <t> time"
+#define   MCHelpSwTO         L"\n  to[mcao]<t>   Process files older than <t> time"
+#define   MCHelpSwTA         L"\n  ta[mcao]<d>   Process files modified after <d> YYYYMMDDHHMMSS date"
+#define   MCHelpSwTB         L"\n  tb[mcao]<d>   Process files modified before <d> YYYYMMDDHHMMSS date"
+#define   MCHelpSwTS         L"\n  ts[m,c,a,p]   Save or restore time (modification, creation, access, preserve)"
 #define   MCHelpSwU          L"\n  u             Update files"
 #define   MCHelpSwV          L"\n  v             Create volumes with size autodetection or list all volumes"
 #define   MCHelpSwVUnr       L"\n  v             List all volumes"
@@ -203,7 +204,6 @@
 #define   MErrOpenFile       L"file"
 #define   MAddNoFiles        L"\nWARNING: No files"
 #define   MMdfEncrSol        L"\n%s: encrypted"
-#define   MCannotMdfEncrSol  L"\nCannot modify solid archive containing encrypted files"
 #define   MAddAnalyze        L"\nAnalyzing archived files: "
 #define   MRepacking         L"\nRepacking archived files: "
 #define   MCRCFailed         L"\n%-20s - checksum error"
@@ -342,7 +342,7 @@
 #define   MFAT32Size         L"\nWARNING: FAT32 file system does not support 4 GB or larger files"
 #define   MErrChangeAttr     L"\nWARNING: Cannot change attributes of %s"
 #define   MWrongSFXVer       L"\nERROR: default SFX module does not support RAR %d.%d archives"
-#define   MCannotEncName     L"\nCannot encrypt archive already containing encrypted files"
+#define   MHeadEncMismatch   L"\nCannot change the header encryption mode in already encrypted archive"
 #define   MCannotEmail       L"\nCannot email the file %s"
 #define   MCopyrightS        L"\nRAR SFX archive" 
 #define   MSHelpCmd          L"\n\n<Commands>" 
@@ -352,14 +352,15 @@
 #define   MRecVolLimit       L"\nTotal number of usual and recovery volumes must not exceed %d"
 #define   MVolumeNumber      L"volume %d"
 #define   MCannotDelete      L"\nCannot delete %s"
-#define   MRecycleFailed     L"\nCannot move some files and folders to Recycle Bin"
+#define   MRecycleFailed     L"\nCannot move some files and directories to Recycle Bin"
 #define   MCalcCRC           L"\nCalculating the checksum"
-#define   MTooLargeSFXArc    L"\nWARNING: Too large SFX archive. Windows cannot run the executable file exceeding 4 GB."
+#define   MTooLargeSFXArc    L"\nToo large SFX archive. Windows cannot run the executable file exceeding 4 GB."
 #define   MCalcCRCAllVol     L"\nCalculating checksums of all volumes."
 #define   MNotEnoughDisk     L"\nERROR: Not enough disk space for %s."
 #define   MNewerRAR          L"\nYou may need a newer version of RAR."
 #define   MUnkEncMethod      L"\nUnknown encryption method in %s"
 #define   MWrongPassword     L"\nThe specified password is incorrect."
+#define   MWrongFilePassword L"\nIncorrect password for %s"
 #define   MAreaDamaged       L"\nCorrupt %d bytes at %08x %08x"
 #define   MBlocksRecovered   L"\n%u blocks are recovered, %u blocks are relocated"
 #define   MRRDamaged         L"\nRecovery record is corrupt."
@@ -379,3 +380,8 @@
 #define   MNeedAdmin         L"\nYou may need to run RAR as administrator"
 #define   MDictOutMem        L"\nNot enough memory for %d MB compression dictionary, changed to %d MB."
 #define   MUseSmalllerDict   L"\nPlease use a smaller compression dictionary."
+#define   MOpenErrAtime      L"\nYou may need to remove -tsp switch to open this file."
+#define   MErrReadInfo       L"\nChoose 'Ignore' to continue with the already read file part only, 'Ignore all' to do it for all read errors, 'Retry' to repeat read and 'Quit' to abort."
+#define   MErrReadTrunc      L"\n%s is archived incompletely because of read error.\n"
+#define   MErrReadCount      L"\n%u files are archived incompletely because of read errors."
+#define   MDirNameExists     L"\nDirectory with such name already exists"
