@@ -26,7 +26,7 @@ Unpack::Unpack(ComprDataIO *DataIO)
   UnpSomeRead=false;
 #ifdef RAR_SMP
   MaxUserThreads=1;
-  UnpThreadPool=CreateThreadPool();
+  UnpThreadPool=NULL;
   ReadBufMT=NULL;
   UnpThreadData=NULL;
 #endif
@@ -52,13 +52,24 @@ Unpack::~Unpack()
   if (Window!=NULL)
     free(Window);
 #ifdef RAR_SMP
-  DestroyThreadPool(UnpThreadPool);
+  delete UnpThreadPool;
   delete[] ReadBufMT;
   delete[] UnpThreadData;
 #endif
 }
 
 
+#ifdef RAR_SMP
+void Unpack::SetThreads(uint Threads)
+{
+  // More than 8 threads are unlikely to provide noticeable gain
+  // for unpacking, but would use the additional memory.
+  MaxUserThreads=Min(Threads,8);
+  UnpThreadPool=new ThreadPool(MaxUserThreads);
+}
+#endif
+
+
 void Unpack::Init(size_t WinSize,bool Solid)
 {
   // If 32-bit RAR unpacks an archive with 4 GB dictionary, the window size
